;**************************************************************************
;*
;* Title:		fastAVRlink
;* Version:		1.2
;* Last updated:	00.09.24
;* Target:		AT90S2313
;*
;* Support E-mail:	rlievin@mail.com
;*
;* Code Size		:?? words
;* Low Register Usage	:?
;* High Register Usage	:?
;* Interrupt Usage	:2 interrupts (RXC & DRE)
;*
;* DESCRIPTION
;*
;* This program clones the Grey TI Graph Link cable of Texas Intruments.
;* Version 1.1: UART used with 2 buffers and up to 38400 bauds.
;* Version 1.2: UART used with 2 buffers and up to 57600 bauds with
;* hardware flow control.
;*
;**************************************************************************

.include "2313def.inc"

;***** WatchDog settings

;.equ	WDP=4	;0.24s at 5.0V (0.75s at 3.0V)
.equ	WDP=5	;0.49s at 5.0V (1.5s at 3.0V)
;.equ	WDP=6	;0.97s at 5.0V


;**************************************************************************
;*
;*	VECTOR TABLE - EXECUTION STARTS HERE
;*
;**************************************************************************
	.cseg

	.org 	$0000
	rjmp	RESET			;Reset handler

	.org 	INT0addr
	rjmp	EXT_INT0		;External interrupt 0 handler

	.org 	INT1addr
	rjmp	EXT_INT1		;External interrupt 0 handler

	.org 	ICP1addr
	rjmp	TIM_CAPT1		;Timer/counter1 capture event handler

	.org	OC1addr
	rjmp	TIM_COMP1		;Timer/counter1 compare match handler

	.org	OVF1addr
	rjmp	TIM_OVF1		;Timer/counter1 overflow handler

	.org	OVF0addr
	rjmp	TIM_OVF0		;Timer/counter0 coverflow handler

	.org	URXCaddr
	rjmp	UART_RXC		;UART, RX complete handler

	.org	UDREaddr
	rjmp	UART_DRE		;UART, Data register empty handler

	.org	UTXCaddr
	rjmp	UART_TXC		;UART, TX complete handler

	.org ACIaddr
	rjmp	ANA_COMP		;Analog comparator handler

; unused vectors: return of interrupt

EXT_INT0:
EXT_INT1:
TIM_CAPT1:
TIM_COMP1:
TIM_OVF1:
TIM_OVF0:
;UART_RXC:
;UART_DRE:
UART_TXC:
ANA_COMP:
	reti

;***** Include subroutines

.include "rs232buf.asm"	:to use RS232 subroutines (buffer)
.include "ti_prot.asm"	;to use TI subroutines


;***** Initialization (common part) 

        .def   	init	=r16
	.def	data	=r17

RESET:  
	ldi	init,low(RAMEND)	;init Stack Pointer	
	out	SPL,init

	ldi	init,0b00000010	 	;set port D bit 1 to output (TxD)
	out	DDRD,init
	

	ldi	init,0b00000100		;set port D bit 3 to output (CTS)
	out	DDRB,init
	clr 	init
	out	PORTB,init		;all output pins at 0

	ldi	init,N			;init the baud rate generator
	out	UBRR,init
	sbi	UCR,TXEN		;transmitter enable
	sbi	UCR,RXEN		;receiver enable
	
wait:	rcall	check			;wait that red=1 & white=1
	brtc	wait			;(in case of watchdog reset)

	ldi	scratch,(1<<WDE) + WDP	;load the watchdog prescaler 
	out	WDTCR,scratch		;and enable watchdog

	rcall	rs_init			;init rs232 routines
	sei				;enable interrupts
	cbi	PORTD,CTS		;CTS = 1 (enable)

;**************************************************************************
;*
;*      Main program, starts here
;*
;**************************************************************************

poll:
	cpi	rx_counter,0	;if buffer is not empty
	brne	rs2ti		;then send a byte to TI calculator
	rcall	check		;check the status of red/white lines
	brtc	ti2rs		;if receiving TI byte, send it to RS232
	rjmp	poll		;else keep on polling	
;
rs2ti:	
	rcall	rs_get		;retrieve RS232 byte
	mov	ti_reg, rs_data
	rcall	wti_put		;and send it to TI
	rjmp	poll
;
ti2rs:	
	rcall	wti_get		;retrieve byte from TI
wait2:
	cpi	tx_counter,TX_BUF_SIZE	;if buffer full
	breq	wait2			;waits
	mov	rs_data,ti_reg
	rcall	rs_put			;and send byte to serial
	rjmp	poll
;
.exit

; a simple test program for TI92
ti_sub:	
	ldi	ti_reg,0x09
	rcall	ti_put
	ldi	ti_reg,0x68
	rcall	ti_put
	ldi	ti_reg,0x00
	rcall	ti_put
	ldi	ti_reg,0x00
	rcall	ti_put
	rcall	ti_get
	rcall	ti_get
	rcall	ti_get
	rcall	ti_get
jmp:	rjmp	jmp
