/*  ASU - fastAVRlink External Setup Utility
 *  Copyright (C) 2000  Romain Lievin
 *   This program was originally written by { Martin Ohsmann, 10.4. 1997 }
 *   I translated it into C, ported to Linux and adapted for programming
 *   AT90S2313 also.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <math.h>

#include "lowlevel.h"
#include "avr_fnct.h"

/* round to the nearest integer */
double rnd(double x)
{
  if( (x - floor(x)) < 0.5)
    return floor(x);
  else
    return ceil(x);
}

/* Global vars */
char com_name[MAXCHARS] = "com2";
int avr_type = 1;					// AT90S1200 by default

/* Main function */
int scan_cmdline(int argc, char **argv);

int main(int argc, char **argv)
{
  int c;
  char s[256];
  int i;
  float xtal;
  int data;
  char buffer[256];

  /* Choose COM port */  
  if(!strcmp(com_name, "com1")) com_adr=COM1ADR;
  else if(!strcmp(com_name, "com2")) com_adr=COM2ADR;
  else if(!strcmp(com_name, "com3")) com_adr=COM3ADR;
  else if(!strcmp(com_name, "com4")) com_adr=COM4ADR;
  
  printf("\n\n");
  scan_cmdline(argc, argv);
  
  /* Unused ! */
  if(avr_type == 1)
    {
      max_cnt = S1200_MAX_CNT;
      max_eep = S1200_MAX_EEP;
    }
  else
    {
      max_cnt = S2313_MAX_CNT;
      max_eep = S2313_MAX_EEP;
    }
  
  /* Init AVR routines */
  setup();
  issue_reset();
  avr_enable();
  issue_reset();
  avr_enable();
  fprintf(stdout, "The fastAVRlink should be in RUN mode. Toggle the jumper in PRG mode (-> remove it).\n");
  fprintf(stdout, "Press the ENTER key when done.\n");
  fflush(stdin);
  getchar();
  issue_reset();
  avr_enable();
  
  /* Display miscellaneous informations */
  printf("\n");
  printf("fastAVRlink Setup Utility, using %s\n", com_name);
  printf("\n");
  printf("** Firmware, version: ");
  for(i=0; i<5; i++)
    printf("%c", avr_read_data(0x08+i));
  printf("\n");
  
  /* Dialogs with the user */
  do
    {
      for(i=0, buffer[0]=' '; i<8; i++)
	{
	  buffer[i] = avr_read_data(0x30+i);
	}
      buffer[i]='\0';
      sscanf(buffer, "%f", &xtal);
      printf("\n");
      printf("** List of parameters **\n");
      
      printf("0. Xtal frequency:\t\t %s MHz\n", buffer);
      data = avr_read_data(0x0);
      printf("1. Link speed:\t\t\t %i bauds\n", 
	     10*(int)rnd((xtal*100000/(16*(data+1)))));
      printf("2. Inter-bit tempo:\t\t %i\n", avr_read_data(0x1));
      printf("3. Buffer size (both Rx & Tx):\t %i bytes\n", avr_read_data(0x2));
      printf("4. Pass trough:\t\t\t %s\n", avr_read_data(0x3) ? "PC->AVR" : 
	     "PC->equipment");
      printf("5. Hardware flow control:\t %s\n", avr_read_data(0x4) ?
	     "enabled" : "disabled");
      printf("6. Watchdog:\t\t\t %s\n", avr_read_data(0x5) ?
	     "enabled" : "disabled");
      printf("\n");

      printf("Enter the parameter to modify ('x' for exiting): ");
      fflush(stdin);
      fscanf(stdin, "%s", s);
      c = toupper(s[0]);
      switch(c)
        {
	  /* XTAL frequency */
	case '0':
	  printf("Enter new crystal frequency (such as 1.8432 for a 1.8432 MHz or 10.0 for a 10MHz xtal): ");
	  if(!scanf("%f", &xtal))
	    {
	      printf("Invalid value, try again !\n");
	      break;
	    }
	  sprintf(buffer, "%2.4f", xtal);
	  if(strlen(buffer) > 8)
	    {
	      printf("Invalid value, try again !\n");
              break;
	    }
	  issue_reset();
	  avr_enable();
	  for(i=0; i<strlen(buffer)+1; i++)
	    avr_pgm_data(0x30+i, buffer[i]);
	  break;
	  /* Link cable speed (depend of the XTAL frequency */
	case '1':
	  printf("Beware: check you have the correct XTAL value !!!\n");
	  printf("Enter new baud-rate (1: 9600, 2:19200, 4:38400, 6:57600): ");
          if(!scanf("%i", &data))
            {
              printf("Invalid value, try again !\n");
              break;
            }
	  data = data * 9600;
	  if(0)
            {
              printf("Invalid value, try again !\n");
              break;
            }
          issue_reset();
          avr_enable();
	  avr_pgm_data(0x0, (int)rnd(xtal*1000000/(16*data))-1);
	  break;
	  /* Inter-bit tempo */
	case '2':
          printf("Enter inter-bit tempo: ");
          if(!scanf("%i", &data))
            {
              printf("Invalid value, try again !\n");
              break;
            }
          issue_reset();
          avr_enable();
          avr_pgm_data(0x1, data);
	  break;
	  /* Buffer size (both Tx & Rx) */
	case '3':
	  printf("Enter the buffer size: ");
          if(!scanf("%i", &data))
            {
              printf("Invalid value, try again !\n");
              break;
            }
	  if((data < 1) || (data > 48))
	    {
	      printf("Invalid value, try again !\n");
              break;
	    }
          issue_reset();
          avr_enable();
          avr_pgm_data(0x2, data);
	break;
	/* Pass through */
	case '4':
	  printf("Enable the pass-through (1) or disable it (0): ");
          if(!scanf("%i", &data))
            {
              printf("Invalid value, try again !\n");
              break;
            }
          if((data != 0) && (data != 1))
            {
              printf("Invalid value, try again !\n");
              break;
            }
          issue_reset();
          avr_enable();
          avr_pgm_data(0x3, data);
	  break;
	  /* Hardware flow contron */
	case '5':
	  printf("Enable flow control (1) or disable it (0): ");
          if(!scanf("%i", &data))
            {
              printf("Invalid value, try again !\n");
              break;
            }
          if((data != 0) && (data != 1))
            {
              printf("Invalid value, try again !\n");
              break;
            }
          issue_reset();
          avr_enable();
          avr_pgm_data(0x4, data);
	  break;
	  /* Watchdog */
	case '6':
	  printf("Enable the watchdog (1) or disable it (0): ");
          if(!scanf("%i", &data))
            {
              printf("Invalid value, try again !\n");
              break;
            }
          if((data != 0) && (data != 1))
            {
              printf("Invalid value, try again !\n");
              break;
            }
          issue_reset();
          avr_enable();
          avr_pgm_data(0x5, data);
	  break;
	default:
	  break;
	}
      
    } 
  while(c != 'X');

  fprintf(stdout, "The fastAVRlink should be in PRG mode. Toggle the jumper in RUN mode (-> put it).\n");
  fprintf(stdout, "Press the ENTER key when done.\n");  
  fflush(stdin);
  getchar();

  return 0;
}

int scan_cmdline(int argc, char **argv)
{
  int cnt;
  char *p;
  char msg[80];
  
  for(cnt=1; cnt<argc; cnt++)
    {
      p=argv[cnt];
      if(*p == '-' ) p++;
      if(*p == '\\') p++;
      strcpy(msg, p);
      
      if(strstr(msg, "type="))
	{
	  if(!strcmp(msg+5, "2"))
	    avr_type = 2;	// 2313
	  else
	    avr_type = 1;	// 1200
	}
      
      if(strstr(msg, "flash="))
	{
	  strcpy(flash_fname, msg+6);
	}
      if(strstr(msg, "eeprom="))
	{
	  strcpy(eeprom_fname, msg+7);
	}
	if(strstr(msg, "com="))
	{
	  strcpy(com_name, msg+4);
	  if(!strcmp(com_name, "com1")) com_adr=COM1ADR;
	  else if(!strcmp(com_name, "com2")) com_adr=COM2ADR;
	  else if(!strcmp(com_name, "com3")) com_adr=COM3ADR;
	  else if(!strcmp(com_name, "com4")) com_adr=COM4ADR;
	}
    }
  printf("AVR type    = %s\n", (avr_type == 1) ? "AT90S1200" : "ATS2313");
  printf("flash_name  = %s\n", flash_fname);
  printf("eeprom_name = %s\n", eeprom_fname);
  printf("COM port    = %s\n", com_name);

  return 0;
}
