/*  avr_prommer - prommer for AVR micro-controllers
 *  Copyright (C) 2000  Romain Lievin
 *   This program was originally written by { Martin Ohsmann, 10.4. 1997 }
 *   in Turbo Pascal.
 *   I translated it into C, ported to Linux and adapted for programming
 *   AT90S2313 also.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdio.h>
#include <stdlib.h>

#include "lowlevel.h"
#include "ioports.h"

/* Constants */

/* Previous version */
//#define SCK		1	// SCK is connected to the DTR line (MCR register)
//#define MOSI	2	// MOSI is connected to the RTS line (MCR register)
//#define MISO	32	// MISO is connected to the DSR line (MSR register)

/* New versioon */
#define SCK		1	// SCK is connected to the DTR line (MCR register)
#define MOSI	2	// MOSI is connected to the RTS line (MCR register)
#define MISO	16	// MISO is connected to the CTS line (MSR register)

/* Global vars */

int com_adr = COM2ADR;
byte mcr_shadow;
long t100us, t500us, t100ms, t10ms;

/* Low level functions */

void sndbyt(byte b)
{
  while( (inb (com_adr+LSR) & 32) == 0) {}
  outb (b, com_adr+TXBUF);
}

byte getbyt(void)
{
  while( (inb(com_adr+LSR) & 1) == 0) {}
  return inb(com_adr+RXBUF);
}

void setcom(int speed)
{
  long f=115200;
  int n_div, div_high, div_low;

  ioperm(com_adr,7,1);
  n_div = f / speed;
  div_low = n_div & 0x00FF;
  div_high = (n_div & 0xFF00) / 256;
  outb (0, com_adr + IER);
  outb (0x83, com_adr + LCR);
  outb (div_high, com_adr + DIVMSB);
  outb (div_low, com_adr + DIVLSB);
  outb (0x03, com_adr + LCR);
}

void wtk(long k)
{
  long j;
  
  for(j=0; j<k; j++) {}
}

void out_clk(int b)
{
  if((b & 1) == 0)
    mcr_shadow |= SCK;
  else
    mcr_shadow &= ~SCK;

  outb(mcr_shadow, com_adr + MCR);
}

void out_dat(int b)
{
  if((b & 1) == 0)
    mcr_shadow |= MOSI;
  else
    mcr_shadow &= ~MOSI;
  
  outb(mcr_shadow, com_adr + MCR);
}

void setup(void)
{
  int ok, k1;
  double k1r;

  printf("Setup...\n");
  mcr_shadow = 0;
  setcom(BITRATE);
  out_clk(0);
  out_dat(0);
  k1r=1;
  ok=FALSE;
  
  while(ok == FALSE)
    {
      k1r *= 1.05;
      k1=round(k1r);
      sndbyt(0xF0);
      wtk(k1);
      if( (inb(com_adr + LSR) & 64) != 0)
		ok = TRUE;
      while( (inb(com_adr + LSR) & 64) == 0) {}
    }
  printf("Calibration value: <%i> Done.\n", k1);
  t100us = round(1.0 * k1 * BITRATE / 10 * 0.0001);
  t500us = round(1.0 * k1 * BITRATE / 10 * 0.0005);
  t100ms = round(1.0 * k1 * BITRATE / 10 * 0.1);
  t10ms  = round(1.0 * k1 * BITRATE / 10 * 0.01);
  setcom(300);
}

void issue_reset(void)
{
  sndbyt(0xF0);
  wtk(t100ms);
}

int get_dat(void)
{
  byte y;

  y=inb(com_adr + MSR);
  if((y & MISO) == 0)
    return 1;
  else
    return 0;
}

void send_byte(byte b)
{
  int k;
  byte bit;

  for(k=8; k>0; k--)
    {
      wtk(t100us);
      bit=(b >> (k-1)) & 1;
      out_dat(bit);
      wtk(t100us);
      out_clk(1);
      wtk(t100us);
      out_clk(0);
    }
}

byte get_byte(void)
{
  int k;
  byte b=0;

  for(k=8; k>0; k--)
    {
      wtk(t100us);
      out_clk(1);
      wtk(t100us);
      b=b | (get_dat() << (k-1));
      out_clk(0);
    }
  return b;
}
